// C Source File
// Created 10/13/01; 11:22:41 AM

#define USE_TI89              // Produce .89z File
//#define USE_TI92PLUS
#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files
#include "pegs.h"


/*****************
**global constants
******************/
#define SPRITE_HEIGHT		8
#define VERSION					"1.1"
#define TSPACE					12
#define LSPACE					45
#define KEY_LEFT				338
#define KEY_RIGHT				344
#define KEY_UP					337
#define KEY_DOWN				340
#define KEY_DIAMOND_Z		16474
#define KEY_DIAMOND_N		16438

#define false						0
#define true						1
#define KEY_HOME				277

/********************
**global variables
**********************/

static unsigned char peg_ini[]=	//initial peg sprite, 8x8
 {	0x18,0x3c,0x7e,0xff,0xff,0x7e,0x3c,0x18 }; 
 
static unsigned char peg[]=		//peg sprite
	{	0x00,0x18,0x3c,0x7e,0x7e,0x3c,0x18,0x00	};
	
int undo1=0,undo2=0,undo3=0,undo4=0;


/***************************
**functions
****************************/
void drawbox(int x,int y){//draw a box around a peg
	x*=10;
	y*=10;
	y+=TSPACE;
	x+=LSPACE;
	DrawLine(x-1,y-1,x-1,y+8,A_XOR);
	DrawLine(x,y-1,x+7,y-1,A_XOR);
	DrawLine(x+8,y-1,x+8,y+8,A_XOR);
	DrawLine(x,y+8,x+7,y+8,A_XOR);
};

int in(int x,int y){ //check if a peg exists
	return ((x>=0)+(y>=0)+(x<=6)+(y<=6)+(x>=2)+(x<=4)+(y>=2)+(y<=4))>=7;
	
};

int jump(int x,int y,int xdir,int ydir)
{
int xm,ym,xf,yf;

	if(!GetPix(x*10+LSPACE+3,y*10+TSPACE+3)) return false;	//if peg doesn't exist, you can't jump
	xm=x+xdir;
	ym=y+ydir;
	if(!in(xm,ym)) return false;  //if there's not a space to jump, you can't jump
	if(!GetPix(xm*10+LSPACE+3,ym*10+TSPACE+3)) return false; //if there's nothing to jump, you can't jump
	xf=xm+xdir;
	yf=ym+ydir;
	if(!in(xf,yf)) return false;	//if there's nowhere to land you cant jump
	if(GetPix(xf*10+LSPACE+3,yf*10+TSPACE+3)) return false; //if landing place is occupied, you can't jump
	//if you made it this far, its ok to jump
	Sprite8(x*10+LSPACE,y*10+TSPACE,SPRITE_HEIGHT,peg,LCD_MEM,SPRT_XOR); //remove first (boxed) peg
	Sprite8(xm*10+LSPACE,ym*10+TSPACE,SPRITE_HEIGHT,peg,LCD_MEM,SPRT_XOR); //remove second (jumped) peg
	Sprite8(xf*10+LSPACE,yf*10+TSPACE,SPRITE_HEIGHT,peg,LCD_MEM,SPRT_XOR);//place final (new) peg
	undo1=xf;
	undo2=yf;
	undo3=xdir;
	undo4=ydir;
	return true;
};

void box(int x1,int y1,int x2,int y2)
{
	DrawLine(x1,y1,x1,y2,A_NORMAL);
	DrawLine(x2,y1,x2,y2,A_NORMAL);
	DrawLine(x1,y1,x2,y1,A_NORMAL);
	DrawLine(x1,y2,x2,y2,A_NORMAL);
};


/*********************************************
***play()
***	checks if play is possible
**********************************************/

void play(void)
{	
	int c1=0,c2=0,i,j;
	int number=0;
	
	for(i=0;(i<=6);i++){
		if ((i<=1)|(i>=5)){
			for(c1=0,c2=0,j=2;j<=4;j++){
				c1<<=1;
				c1+=GetPix(i*10+LSPACE+3,j*10+TSPACE+3);
				c2<<=1;
				c2+=GetPix(j*10+LSPACE+3,i*10+TSPACE+3);
			};//endfor(j)
			if ((c1==3)|(c1==6)|(c2==3)|(c2==6)) return;
		}//endif
		else //if it's a big row
		{
		for(c1=0,c2=0,j=0;j<=6;j++){
			c1<<=1;
			c1+=GetPix(i*10+LSPACE+3,j*10+TSPACE+3);
			c1&=7;
			c2<<=1;
			c2+=GetPix(j*10+LSPACE+3,i*10+TSPACE+3);
			c2&=7;
			if ((j>1)&((c1==3)|(c1==6)|(c2==3)|(c2==6))) return;
		
		};//endfor(j)
		};//end else (if its a big row)	
	}; //endfor(i)

	/*****the following is an alternate way to check for remaining plays.  I don't think its as fast, its definitely not
	******as cool
	
	
		for (i=0;(i<=6);i++){
			for (j=0;(j<=6);j++){
				if (in(i,j)){
					if (GetPix(i*10+LSPACE+3,j*10+TSPACE+3))	c1++;
					else {if (c1>1)	return;
						c1=0;	};//end else
					if (GetPix(j*10+LSPACE+3,i*10+TSPACE+3))	c2++;
					else	{if (c2>1) return;
						c2=0;	};//end else
					
				}//end if (in(i,j))
				else { if((c1==2)|(c2==2)) return;
					c1=0;	c2=0;
				};//end else
			};//end for(j)
			if (((c1>1)&(c1<7))|((c2>1)&(c2<7))) return;
			c1=0;c2=0;
		};//end for(i)
		//if it gets to here, the game is OVER
		*/
		undo1=0;
		FontSetSys(F_8x10);
		DrawStr(95,12,"GameOver",A_REVERSE);		
		box(95,11,159,21);
		FontSetSys(F_4x6);
		for (i=0;i<=6;i++){
			for (j=0;j<=6;j++){
				if (in(i,j)&GetPix(i*10+LSPACE+3,j*10+TSPACE+3)) number++;
			};//endfor(j)
		};//endfor(i)
		if (number>1) printf_xy(98,23,"%d pegs remaining",number);
		else printf_xy(112,23,"You win!!!");
};


/*********************
******Main Function
*********************/
void _main(void)
{
	int x=3,y=3;
	int i,j;
	int quit=false;
	FILE *f;

	
	
	/*********************
	***start of execution
	**********************/

	//draw board
start:
	undo1=0;
	ClrScr();
	FontSetSys(F_8x10);
	printf_xy(15,12,"PEGS");
	FontSetSys(F_4x6);
	printf_xy(2,22,"by Nathan Grigg");
	box(1,10,58,28);
  printf_xy(52,95,"Pegs v"VERSION" %c 2001",169);
	printf_xy(101,71,"Arrows to move");
	printf_xy(107,65,"2nd to jump");
	printf_xy(99,77,"%cZ Undo %cN New",127,127);
	box(98,63,158,83);
	
	f=fopen("pegsave","rb");
	if (f!=NULL)
		{if (fgetc(f)!=0) fclose(f),f=NULL;};  //if pegsave exists but is not in good format, pretend it doesn't exist
	
	for (i=0;i<7;i++){
		for (j=0;j<7;j++){
			if (((i<2)+(i>4)+(j<2)+(j>4))!=2){
				Sprite8(i*10+LSPACE,j*10+TSPACE,SPRITE_HEIGHT,peg_ini,LCD_MEM,SPRT_XOR);
			if (f!=NULL){
				if (fgetc(f)!='x') Sprite8(i*10+LSPACE,j*10+TSPACE,SPRITE_HEIGHT,peg,LCD_MEM,SPRT_XOR);
			};//end if f!=null
			};//end if i<2...
		};//end for (j)
	};//end for(i)
	if (f==NULL) Sprite8(x*10+LSPACE,y*10+TSPACE,SPRITE_HEIGHT,peg,LCD_MEM,SPRT_XOR);
	else fclose(f);
	remove("pegsave");

	drawbox(x,y);//box the piece
	while ((!quit)){		//get keyboard input
		switch(ngetchx()){
			case KEY_LEFT:
				drawbox(x,y);
			  x--;
			  if(!in(x,y)) x++;
			  drawbox(x,y);
			  break;

			case KEY_RIGHT:
  			drawbox(x,y);
			  x++;
			  if(!in(x,y)) x--;
			  drawbox(x,y);
			  break;
			  
			case KEY_UP:
				drawbox(x,y);
			  y--;
			  if(!in(x,y)) y++;
			  drawbox(x,y);
			  break;
			  			
			case KEY_DOWN:
				drawbox(x,y);
			  y++;
			  if(!in(x,y)) y--;
			  drawbox(x,y);			
			  break;
			
			case KEY_LEFT+4096:
				if (jump(x,y,-1,0))	//jump a piece and remove it
					{drawbox(x,y);
					 drawbox(x-=2,y);							//if jump is successful,subtract two from x and fix box
					 play();
					};
				break;

			case KEY_RIGHT+4096:
				if (jump(x,y,1,0)){
					drawbox(x,y);
					drawbox(x+=2,y);
					play();
				};
				break;
				
			case KEY_UP+4096:
				if (jump(x,y,0,-1)){
					drawbox(x,y);
					drawbox(x,y-=2);
					play();
				};
				break;
			
			case KEY_DOWN+4096:
				if (jump(x,y,0,1)){
					drawbox(x,y);
					drawbox(x,y+=2);
					play();
				};
				break;
				
			case KEY_DIAMOND_Z:
				if(undo1)//if undo contains a value
				{
					drawbox(x,y);//erase current box
					x=undo1;
					y=undo2;
					Sprite8(x*10+LSPACE,y*10+TSPACE,SPRITE_HEIGHT,peg,LCD_MEM,SPRT_XOR);	//invert first peg
					Sprite8((x-=undo3)*10+LSPACE,(y-=undo4)*10+TSPACE,SPRITE_HEIGHT,peg,LCD_MEM,SPRT_XOR); //invert next peg
					Sprite8((x-=undo3)*10+LSPACE,(y-=undo4)*10+TSPACE,SPRITE_HEIGHT,peg,LCD_MEM,SPRT_XOR); //invert next peg
					drawbox(x,y);  //draw new box
					};
					break;

			
			case KEY_DIAMOND_N:
			  undo1=0;
			  x=3;
			  y=3;
			  goto start;
			  break;
			  
			case KEY_ESC:
			case KEY_HOME:
			 if (undo1){
				f=fopen("pegsave","wb");
				fputc(0,f);
				for (i=0;i<7;i++){
					for (j=0;j<7;j++){
						if (in(i,j)){
						  if (GetPix(i*10+LSPACE+3,j*10+TSPACE+3)) fputc('x',f); else fputc('o',f);
					};};
				};
				fputc(0,f);
				fputc(STR_TAG,f);
				fclose(f);
				};
			case KEY_QUIT:
				quit=true;
				break;
					

		};//end switch		
	};//end while
	};//end _main
	




